@@ -1,5 +1,5 @@
 /* This file is part of GNU Pies.
-   Copyright (C) 2009-2019 Sergey Poznyakoff
+   Copyright (C) 2009-2025 Sergey Poznyakoff
 
    GNU Pies is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <gettext.h>
 #include <netinet/in.h>
 #include <sys/un.h>
+#include <sys/socket.h>
 #include <grecs.h>
 
 #if defined HAVE_SYSCONF && defined _SC_OPEN_MAX
@@ -29,6 +30,8 @@
 #else
 # define getmaxfd() 256
 #endif
+
+void pies_close_fds (int minfd);
 
 void mf_proctitle_init (int argc, char *argv[], char *env[]);
 void mf_proctitle_format (const char *fmt, ...);
@@ -84,6 +87,7 @@ void netrc_scan (struct pies_url *url);
 
 
 /* pp.c */
+void pp_init (char const *path);
 void pp_add_option (const char *opt, const char *arg);
 char *pp_command_line (void);
 
