@@ -1,5 +1,5 @@
 /* This file is part of GNU Pies.
-   Copyright (C) 2007-2022 Sergey Poznyakoff
+   Copyright (C) 2007-2025 Sergey Poznyakoff
 
    GNU Pies is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -705,7 +705,7 @@ ctlio_finalize_reply (struct ctlio *io)
       };
 
       ctlio_adjust_format (io, &fmt);
-      json_format_value (io->output.reply, &fmt);
+      json_value_format (io->output.reply, &fmt, 0);
       size = ctlbuf_rdsize (&tmpbuf);
       if (size)
 	{
@@ -836,7 +836,7 @@ ctlio_authenticate (struct ctlio *io)
     {
       size_t len = strlen (val);
       char *data = NULL;
-      size_t datalen = 0;
+      idx_t datalen = 0;
       if (len > 6 && memcmp (val, "Basic ", 6) == 0)
 	{
 	  if (base64_decode_alloc (val + 6, len - 6, &data, &datalen))
@@ -1133,17 +1133,19 @@ ctlio_do_command (struct ctlio *io)
 
   if (io->input.input_content_length)
     {
+      char const *errmsg;
       if (json_query)
 	{
 	  ctlio_reply (io, 400, "JSON query supplied twice");
 	  return;
 	}
 	    
-      json = json_parse_string (ctlbuf_peek (&io->input.ibuf),
-				ctlbuf_rdsize (&io->input.ibuf));
+      json = grecs_json_parse_string (ctlbuf_peek (&io->input.ibuf),
+				      ctlbuf_rdsize (&io->input.ibuf),
+				      &errmsg, NULL);
       if (!json)
 	{
-	  ctlio_reply (io, 400, "JSON error: %s", json_err_diag);
+	  ctlio_reply (io, 400, "JSON error: %s", errmsg);
 	  return;
 	}
     }
@@ -1156,10 +1158,12 @@ ctlio_do_command (struct ctlio *io)
 	}
       else
 	{
-	  json = json_parse_string (json_query, strlen (json_query));
+	  char const *errmsg;
+	  json = grecs_json_parse_string (json_query, strlen (json_query),
+					  &errmsg, NULL);
 	  if (!json)
 	    {
-	      ctlio_reply (io, 400, "JSON error: %s", json_err_diag);
+	      ctlio_reply (io, 400, "JSON error: %s", errmsg);
 	      return;
 	    }
 	}
@@ -1629,7 +1633,7 @@ pcond_conv_component (struct pcond_node *node, struct json_value *arg,
       ctlio_reply (io, 400, "component must be string");
       return -1;
     }
-  node->v.tag = arg->v.s;
+  node->v.tag = arg->s;
   return 0;
 }
 
@@ -1644,7 +1648,7 @@ pcond_conv_n (struct pcond_node *node, struct json_value *arg,
       ctlio_reply (io, 400, "%s must be string", what);
       return -1;
     }
-  n = array_index (ar, arg->v.s);
+  n = array_index (ar, arg->s);
   if (n == -1)
     {
       ctlio_reply (io, 400, "bad %s", what);
@@ -1701,13 +1705,13 @@ pcond_conv_binary (struct pcond_node *node, struct json_value *arg,
 {
   size_t i, n;
   
-  if (arg->type != json_arr)
+  if (arg->type != json_array)
     {
       ctlio_reply (io, 400, "arguments to binary opcode must be array");
       return -1;
     }
 
-  n = json_array_size (arg);
+  n = json_array_length (arg);
   node->v.arg.v = grecs_calloc (n, sizeof (node->v.arg.v[0]));
   node->v.arg.c = n;
 				
@@ -1773,7 +1777,7 @@ object_to_cond (struct ctlio *io, struct json_value *obj,
       return -1;
     }
 
-  type = pcond_conv_find (op->v.s);
+  type = pcond_conv_find (op->s);
   if (type == -1)
     {
       ctlio_reply (io, 400, "bad conditional: unknown opcode");
@@ -1810,12 +1814,12 @@ json_to_pcond (struct ctlio *io, struct json_value *val,
 	break;
 	
       case json_bool:
-	*pnode = pcond_node_alloc (val->v.b ? pcond_true : pcond_false);
+	*pnode = pcond_node_alloc (val->b ? pcond_true : pcond_false);
 	break;
 	
       case json_number:
       case json_string:
-      case json_arr:
+      case json_array:
 	ctlio_reply (io, 400, "bad conditional");
 	return -1;
 	
@@ -2205,8 +2209,8 @@ res_runlevel (struct ctlio *io, enum http_method meth,
 	{
 	  io->output.reply = json_reply_create ();
 	  io->code = 200;
-	  if (strlen (val->v.s) == 1
-	      && sysvinit_set_runlevel (val->v.s[0]) == 0)
+	  if (strlen (val->s) == 1
+	      && sysvinit_set_runlevel (val->s[0]) == 0)
 	    {
 	      json_object_set_string (io->output.reply, "status", "OK");
 	    }
@@ -2339,7 +2343,7 @@ conf_add_file (struct ctlio *io, struct json_value *json)
       ctlio_reply (io, 400, NULL);
       return;
     }
-  synt = str_to_config_syntax (v->v.s);
+  synt = str_to_config_syntax (v->s);
   if (!synt)
     {
       ctlio_reply (io, 400, NULL);
@@ -2351,7 +2355,7 @@ conf_add_file (struct ctlio *io, struct json_value *json)
       return;
     }
 
-  config_file_add (synt, v->v.s);
+  config_file_add (synt, v->s);
   ctlio_reply (io, 201, NULL);
 }
 
@@ -2361,20 +2365,20 @@ conf_delete_files (struct ctlio *io, struct json_value *json)
   io->code = 200;
   io->output.reply = json_reply_create ();
 
-  if ((json->type == json_bool && json->v.b == 1)
-      || (json->type == json_arr && json_array_size (json) == 0))
+  if ((json->type == json_bool && json->b == 1)
+      || (json->type == json_array && json_array_length (json) == 0))
     {
       config_file_remove_all ();
       json_object_set_string (io->output.reply, "status", "OK");
       json_object_set_string (io->output.reply, "message",
 			      "file list cleared");
     }
-  else if (json->type == json_arr)
+  else if (json->type == json_array)
     {
       size_t i, n, fails = 0;
       struct json_value *reply;
 
-      n = json_array_size (json);
+      n = json_array_length (json);
 
       /* Check request */
       for (i = 0; i < n; i++)
@@ -2399,7 +2403,7 @@ conf_delete_files (struct ctlio *io, struct json_value *json)
 	  
 	  json_array_get (json, i, &val);
 	  
-	  rc = config_file_remove (val->v.s);
+	  rc = config_file_remove (val->s);
 	  if (rc)
 	    ++fails;
 	  json_array_append (reply, json_new_bool (!rc));
