@@ -45,6 +45,7 @@
 #include <ctype.h>
 
 #include <grecs.h>
+#include <grecs/jsonparse.h>
 #include <wordsplit.h>
 
 #include "progname.h"
@@ -199,24 +200,25 @@ enum pies_comp_mode
 #define PIES_COMP_DEFAULT 0
 #define PIES_COMP_MASK(m) (1 << ((m)))
 
-#define CF_DISABLED    0x001 /* The componenet is disabled */
-#define CF_PRECIOUS    0x002 /* The component is precious (should not
-				be disabled) */
-#define CF_WAIT        0x004 /* Wait for the component instance to
-				terminate. */
-#define CF_TCPMUX      0x008 /* A plain TCPMUX service */
-#define CF_TCPMUXPLUS  0x010 /* A TCPMUX-plus service, i.e. pies
-				must emit a '+' response before starting
-				it */
-#define CF_INTERNAL    0x020 /* An internal inetd service */
-#define CF_SOCKENV     0x040 /* Component wants socket information in
-				the environment */
-#define CF_RESOLVE     0x080 /* Resolve IP addresses */
-#define CF_SIGGROUP    0x100 /* Send signals to the process group */
-#define CF_NULLINPUT   0x200 /* Provide null input stream */
-#define CF_SHELL       0x400 /* Invoke via sh -c */
-#define CF_EXPANDENV   0x800 /* Expand environment variables in the command
-				line */
+#define CF_DISABLED    0x0001 /* The componenet is disabled */
+#define CF_PRECIOUS    0x0002 /* The component is precious (should not
+			 	 be disabled) */
+#define CF_WAIT        0x0004 /* Wait for the component instance to
+				 terminate. */
+#define CF_TCPMUX      0x0008 /* A plain TCPMUX service */
+#define CF_TCPMUXPLUS  0x0010 /* A TCPMUX-plus service, i.e. pies
+				 must emit a '+' response before starting
+				 it */
+#define CF_INTERNAL    0x0020 /* An internal inetd service */
+#define CF_SOCKENV     0x0040 /* Component wants socket information in
+				 the environment */
+#define CF_RESOLVE     0x0080 /* Resolve IP addresses */
+#define CF_SIGGROUP    0x0100 /* Send signals to the process group */
+#define CF_NULLINPUT   0x0200 /* Provide null input stream */
+#define CF_SHELL       0x0400 /* Invoke via sh -c */
+#define CF_EXPANDENV   0x0800 /* Expand environment variables in the command
+				 line */
+#define CF_DEANSIFY    0x1000 /* Remove ANSI escape sequences. */
 
 #define CF_REMOVE      0xf000 /* Marked for removal */
 
